@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -104,6 +104,9 @@ void AppUiBase::buildMenus()
   addItem(EFileMenu, "Save", "save");
   addItem(EFileMenu, "Save as", "save_as");
   addItem(EFileMenu);
+  addItem(EFileMenu, "Export as PNG", "export_png");
+  addItem(EFileMenu, "Export as EPS", "export_eps");
+  addItem(EFileMenu);
   addItem(EFileMenu, "Insert image", "insert_image");
   addItem(EFileMenu);
   addItem(EFileMenu, "Automatically run Latex", "*auto_latex");
@@ -156,7 +159,7 @@ void AppUiBase::buildMenus()
 
   addItem(EPropertiesMenu);
 
-  startSubMenu(EPropertiesMenu, "Text style");
+  startSubMenu(EPropertiesMenu, "Text style", ESubmenuTextStyle);
   iTextStyleMenu = endSubMenu();
 
   startSubMenu(EPropertiesMenu, "Horizontal alignment");
@@ -208,17 +211,22 @@ void AppUiBase::buildMenus()
 
   // @ means the action can be used while drawing
   // * means the action is checkable (on/off)
-  // Checkable actions work differently in Qt and Win32:
+  // Checkable actions work differently in Qt and Win32/Cocoa:
   //   Qt already toggles the state
-  //   In Win32 the action needs to toggle the state.
+  //   In Win32/Cocoa the action needs to toggle the state.
 
-  addItem(ESnapMenu, "Snap to vertex", "@snapvtx");
-  addItem(ESnapMenu, "Snap to control point", "@snapctl");
-  addItem(ESnapMenu, "Snap to boundary", "@snapbd");
-  addItem(ESnapMenu, "Snap to intersection", "@snapint");
-  addItem(ESnapMenu, "Snap to grid", "@snapgrid");
-  addItem(ESnapMenu, "Angular snap", "@snapangle");
-  addItem(ESnapMenu, "Automatic snap", "@snapauto");
+  addItem(ESnapMenu, "Snap to vertex", "@*snapvtx");
+  addItem(ESnapMenu, "Snap to control point", "@*snapctl");
+  addItem(ESnapMenu, "Snap to boundary", "@*snapbd");
+  addItem(ESnapMenu, "Snap to intersection", "@*snapint");
+  addItem(ESnapMenu, "Snap to grid", "@*snapgrid");
+  addItem(ESnapMenu, "Angular snap", "@*snapangle");
+  addItem(ESnapMenu, "Automatic snap", "@*snapauto");
+  addItem(ESnapMenu);
+  startSubMenu(ESnapMenu, "Grid size", ESubmenuGridSize);
+  iGridSizeMenu = endSubMenu();
+  startSubMenu(ESnapMenu, "Radial angle", ESubmenuAngleSize);
+  iAngleSizeMenu = endSubMenu();
   addItem(ESnapMenu);
   addItem(ESnapMenu, "Set origin && snap", "@set_origin_snap");
   addItem(ESnapMenu, "Hide axes", "@hide_axes");
@@ -269,10 +277,10 @@ void AppUiBase::buildMenus()
   addItem(ELayerMenu);
   addItem(ELayerMenu, "Select all in active  layer",
     "select_in_active_layer");
-  startSubMenu(ELayerMenu, "Select all in layer");
+  startSubMenu(ELayerMenu, "Select all in layer", ESubmenuSelectLayer);
   iSelectLayerMenu = endSubMenu();
   addItem(ELayerMenu, "Move to active layer", "move_to_active_layer");
-  startSubMenu(ELayerMenu, "Move to layer");
+  startSubMenu(ELayerMenu, "Move to layer", ESubmenuMoveLayer);
   iMoveToLayerMenu = endSubMenu();
 
   addItem(EViewMenu, "Next view", "next_view");
@@ -303,7 +311,7 @@ void AppUiBase::buildMenus()
   addItem(EPageMenu, "Edit notes", "edit_notes");
   addItem(EPageMenu, "Page sorter", "page_sorter");
   addItem(EPageMenu);
-#ifdef IPEUI_WIN32
+#ifndef IPEUI_QT
   // In Qt these are created using "toggleViewAction()"
   addItem(EPageMenu, "Notes", "@*toggle_notes");
   addItem(EPageMenu, "Bookmarks", "@*toggle_bookmarks");
@@ -324,7 +332,9 @@ void AppUiBase::buildMenus()
   }
   lua_pop(L, 2); // developer, prefs
 
+#ifndef IPEUI_COCOA
   addItem(EHelpMenu, "&About Ipe", "about");
+#endif
 
   // build Ipelet menu
   lua_getglobal(L, "ipelets");
@@ -368,14 +378,17 @@ void AppUiBase::buildMenus()
 
 // --------------------------------------------------------------------
 
-void AppUiBase::canvasObserverWheelMoved(int degrees)
+void AppUiBase::canvasObserverWheelMoved(double degrees, bool precise)
 {
-  if (degrees > 0)
-    action("wheel_zoom_in");
-  else if (degrees < 0)
-    action("wheel_zoom_out");
-  else
-    // result of a zoom gesture
+  if (degrees != 0.0) {
+    lua_rawgeti(L, LUA_REGISTRYINDEX, iModel);
+    lua_getfield(L, -1, "wheel_zoom");
+    lua_insert(L, -2); // model
+    lua_pushnumber(L, degrees);
+    lua_pushboolean(L, precise);
+    lua_call(L, 3, 0);
+  } else
+    // result of a zoom gesture (Windows only, currently)
     setZoom(iCanvas->zoom());
 }
 
@@ -441,6 +454,11 @@ void AppUiBase::canvasObserverSizeChanged()
 
 // --------------------------------------------------------------------
 
+int AppUiBase::actionInfo(lua_State *L) const
+{
+  return 0; // only Windows will override this
+}
+
 static void call_selector(lua_State *L, int model, String name)
 {
   // calls model selector
@@ -579,7 +597,7 @@ void AppUiBase::showMarksInCombo(const Cascade *sheet)
   sheet->allNames(ESymbol, seq);
   for (uint i = 0; i < seq.size(); ++i) {
     String s = stripMark(seq[i]);
-    if (!s.isEmpty()) {
+    if (!s.empty()) {
       addCombo(EUiMarkShape, s);
       iComboContents[EUiMarkShape].push_back(s);
     }
