@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -83,32 +83,6 @@ QIcon prefsColorIcon(Color color)
 
 // --------------------------------------------------------------------
 
-IpeApp::IpeApp(lua_State *L0, int &argc, char **argv)
-  : QApplication(argc, argv)
-{
-  L = L0;
-}
-
-bool IpeApp::event(QEvent *event)
-{
-  switch (event->type()) {
-  case QEvent::FileOpen:
-    loadFile(static_cast<QFileOpenEvent *>(event)->file());
-    return true;
-  default:
-    return QApplication::event(event);
-  }
-}
-
-void IpeApp::loadFile(const QString &fileName)
-{
-  lua_getglobal(L, "file_open_event");
-  lua_pushstring(L, fileName.toUtf8());
-  lua_call(L, 1, 0);
-}
-
-// --------------------------------------------------------------------
-
 QAction *AppUi::findAction(const char *name) const
 {
   std::map<String, QAction*>::const_iterator it = iActions.find(name);
@@ -118,18 +92,6 @@ QAction *AppUi::findAction(const char *name) const
     return 0;
 }
 
-// Not used
-int AppUi::actionId(const char *name) const
-{
-  return 0;
-}
-
-// Not used
-int AppUi::actionAlwaysOn(const char *name) const
-{
-  return 0;
-}
-
 void AppUi::addItem(QMenu *m, const QString &title, const char *name)
 {
   // bool canUseWhileDrawing = false;
@@ -153,23 +115,6 @@ void AppUi::addItem(QMenu *m, const QString &title, const char *name)
     a->setShortcut(QKeySequence(s));
     QString tt = title + " [" + s + "]";
     a->setToolTip(tt);
-  } else if (lua_istable(L, -1)) {
-    int no = lua_rawlen(L, -1);
-    QList<QKeySequence> kl;
-    QString prim;
-    for (int i = 1; i <= no; ++i) {
-      lua_rawgeti(L, -1, i);
-      if (lua_isstring(L, -1)) {
-	QString s = lua_tostring(L, -1);
-	kl.append(QKeySequence(s));
-	if (prim.isEmpty())
-	  prim = s;
-      }
-      lua_pop(L, 1); // pop string
-    }
-    a->setShortcuts(kl);
-    QString tt = title + " [" + prim + "]";
-    a->setToolTip(tt);
   }
   a->setIcon(prefsIcon(name));
   lua_pop(L, 2);
@@ -215,7 +160,7 @@ void AppUi::addItem(int id, const char *title, const char *name)
 static QMenu *submenu = 0;
 static int submenuId = 0;
 
-void AppUi::startSubMenu(int id, const char *name)
+void AppUi::startSubMenu(int id, const char *name, int tag)
 {
   submenuId = id;
   submenu = new QMenu(name);
@@ -237,7 +182,7 @@ Qt::DockWidgetArea getDockSide(lua_State *L, const char *name,
 {
   Qt::DockWidgetArea r = deflt;
   lua_getglobal(L, "prefs");
-  lua_getfield(L, -1, "docks");
+  lua_getfield(L, -1, "tools");
   if (lua_istable(L, -1)) {
     lua_getfield(L, -1, name);
     const char *s = lua_tolstring(L, -1, 0);
@@ -247,7 +192,7 @@ Qt::DockWidgetArea getDockSide(lua_State *L, const char *name,
       r = Qt::RightDockWidgetArea;
     lua_pop(L, 1); // left or right
   }
-  lua_pop(L, 2); // prefs, docks
+  lua_pop(L, 2); // prefs, tools
   return r;
 }
 
@@ -255,16 +200,17 @@ Qt::DockWidgetArea getDockSide(lua_State *L, const char *name,
 
 extern void qt_set_sequence_auto_mnemonic(bool b);
 
+#if QT_VERSION < 0x050000
+AppUi::AppUi(lua_State *L0, int model, Qt::WFlags f)
+#else
 AppUi::AppUi(lua_State *L0, int model, Qt::WindowFlags f)
+#endif
   : QMainWindow(0, f), AppUiBase(L0, model)
 {
   setWindowIcon(prefsIcon("ipe"));
   QMainWindow::setAttribute(Qt::WA_DeleteOnClose);
   setDockOptions(AnimatedDocks); // do not allow stacking properties and layers
 
-  // enable automatic shortcuts on Mac OS X
-  qt_set_sequence_auto_mnemonic(true);
-
   Canvas *canvas = new Canvas(this);
   iCanvas = canvas;
   setCentralWidget(canvas);
@@ -304,6 +250,10 @@ AppUi::AppUi(lua_State *L0, int model, Qt::WindowFlags f)
 	  SLOT(moveToLayerAction(QAction *)));
   connect(iTextStyleMenu, SIGNAL(triggered(QAction *)),
 	  SLOT(textStyleAction(QAction *)));
+  connect(iGridSizeMenu, SIGNAL(triggered(QAction *)),
+	  SLOT(gridSizeAction(QAction *)));
+  connect(iAngleSizeMenu, SIGNAL(triggered(QAction *)),
+	  SLOT(angleSizeAction(QAction *)));
 
   QSignalMapper *comboMap = new QSignalMapper(this);
 
@@ -440,10 +390,14 @@ AppUi::AppUi(lua_State *L0, int model, Qt::WindowFlags f)
   connect(iPathView, SIGNAL(activated(String)), SLOT(action(String)));
   connect(iPathView, SIGNAL(showPathStylePopup(Vector)),
 	  SLOT(showPathStylePopup(Vector)));
-  lo->addWidget(iPathView, 3, 0, 1, 2);
+  lo->addWidget(iPathView, 3, 1, 1, 1);
   // lo->setRowStretch(3, 1);
   iPropertiesTools->setWidget(wg);
 
+  iModeIndicator = new QLabel();
+  iModeIndicator->setPixmap(prefsPixmap("mode_select"));
+  lo->addWidget(iModeIndicator, 3, 0);
+
   QHBoxLayout *hol = new QHBoxLayout();
   iViewNumber = new QToolButton();
   iPageNumber = new QToolButton();
@@ -496,24 +450,25 @@ AppUi::AppUi(lua_State *L0, int model, Qt::WindowFlags f)
 	  this, SLOT(aboutToShowMoveToLayerMenu()));
   connect(iTextStyleMenu, SIGNAL(aboutToShow()),
 	  this, SLOT(aboutToShowTextStyleMenu()));
+  connect(iGridSizeMenu, SIGNAL(aboutToShow()),
+	  this, SLOT(aboutToShowGridSizeMenu()));
+  connect(iAngleSizeMenu, SIGNAL(aboutToShow()),
+	  this, SLOT(aboutToShowAngleSizeMenu()));
+
+  iSnapIndicator = new QLabel(statusBar());
+  statusBar()->addPermanentWidget(iSnapIndicator, 0);
+  QFont font = iSnapIndicator->font();
+  font.setFamily("Monospace");
+  iSnapIndicator->setFont(font);
 
   iMouse = new QLabel(statusBar());
   findAction("coordinates|points")->setChecked(true);
   statusBar()->addPermanentWidget(iMouse, 0);
-  QFont font = iMouse->font();
-  font.setFamily("Monospace");
   iMouse->setFont(font);
 
   iResolution = new QLabel(statusBar());
   statusBar()->addPermanentWidget(iResolution, 0);
 
-  lua_getglobal(L, "prefs");
-  lua_getfield(L, -1, "retina");
-  int retina = lua_toboolean(L, -1) ? 2 : 1;
-  ipeDebug("setRetina(%d)", retina);
-  iCanvas->setRetina(retina);
-  lua_pop(L, 2); // retina, prefs
-
   iCanvas->setObserver(this);
 }
 
@@ -567,11 +522,39 @@ void AppUi::aboutToShowTextStyleMenu()
   }
 }
 
+void AppUi::aboutToShowGridSizeMenu()
+{
+  iGridSizeMenu->clear();
+  for (auto &name : iComboContents[EUiGridSize]) {
+    QString s = QIpe(name);
+    QAction *a = new QAction(s, iGridSizeMenu);
+    a->setCheckable(true);
+    if (s == iSelector[EUiGridSize]->currentText())
+      a->setChecked(true);
+    iGridSizeMenu->addAction(a);
+  }
+}
+
+void AppUi::aboutToShowAngleSizeMenu()
+{
+  iAngleSizeMenu->clear();
+}
+
 void AppUi::textStyleAction(QAction *a)
 {
   action(String("textstyle|") + IpeQ(a->text()));
 }
 
+void AppUi::gridSizeAction(QAction *a)
+{
+  action(String("gridsize|") + IpeQ(a->text()));
+}
+
+void AppUi::angleSizeAction(QAction *a)
+{
+  action(String("anglesize|") + IpeQ(a->text()));
+}
+
 void AppUi::toolbarModifiersChanged()
 {
   if (iCanvas) {
@@ -642,7 +625,7 @@ void AppUi::setCheckMark(String name, Attribute a)
 
 void AppUi::setNumbers(String vno, bool vm, String pno, bool pm)
 {
-  if (vno.isEmpty()) {
+  if (vno.empty()) {
     iViewNumber->hide();
     iViewMarked->hide();
   } else {
@@ -651,7 +634,7 @@ void AppUi::setNumbers(String vno, bool vm, String pno, bool pm)
     iViewMarked->setCheckState(vm ? Qt::Checked : Qt::Unchecked);
     iViewMarked->show();
   }
-  if (pno.isEmpty()) {
+  if (pno.empty()) {
     iPageNumber->hide();
     iPageMarked->hide();
   } else {
@@ -803,19 +786,18 @@ void AppUi::comboSelector(int id)
 
 static const char * const aboutText =
 "<qt><h2>Ipe %d.%d.%d</h2>"
-"<p>Copyright (c) 1993-2015 Otfried Cheong</p>"
+"<p>Copyright (c) 1993-2016 Otfried Cheong</p>"
 "<p>The extensible drawing editor Ipe creates figures "
 "in Postscript and PDF format, "
 "using LaTeX to format the text in the figures.</p>"
 "<p>Ipe relies on the following fine pieces of software:"
 "<ul>"
-"<li> Pdflatex"
+"<li> Pdftex, Xetex, or Luatex"
 "<li> %s (%d kB used)" // Lua
 "<li> The font rendering library %s"
 "<li> The rendering library Cairo %s / %s"
 "<li> The GUI toolkit Qt %s / %s"
 "<li> The compression library zlib %s"
-"<li> Some code from Xpdf"
 "</ul>"
 "<p>Ipe is released under the GNU Public License.</p>"
 "<p>See <a href=\"http://ipe.otfried.org\">ipe.otfried.org</a>"
@@ -856,8 +838,11 @@ void AppUi::action(String name)
     setWindowState(windowState() ^ Qt::WindowFullScreen);
   } else if (name == "about") {
     aboutIpe();
-  } else
+  } else {
+    if (name.left(5) == "mode_")
+      iModeIndicator->setPixmap(prefsPixmap(name));
     luaAction(name);
+  }
 }
 
 // --------------------------------------------------------------------
@@ -935,6 +920,11 @@ void AppUi::setMouseIndicator(const char *s)
   iMouse->setText(s);
 }
 
+void AppUi::setSnapIndicator(const char *s)
+{
+  iSnapIndicator->setText(s);
+}
+
 void AppUi::explain(const char *s, int t)
 {
   statusBar()->showMessage(QString::fromUtf8(s), t);
