@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -42,8 +42,8 @@
 #include <cstdio>
 #include <cstdlib>
 
-#include <cairo.h>
 #include <windowsx.h>
+#include <gdiplus.h>
 
 #define IDI_MYICON 1
 #define ID_STATUS_TIMER 1
@@ -59,6 +59,8 @@
 #define ID_PATHVIEW 8700
 #define ID_MOVETOLAYER_BASE 8800
 #define ID_SELECTINLAYER_BASE 9000
+#define ID_GRIDSIZE_BASE 9100
+#define ID_ANGLESIZE_BASE 9500
 
 using namespace ipe;
 using namespace ipelua;
@@ -81,11 +83,20 @@ static HBITMAP loadIcon(String action, int w, int h, int r0, int g0, int b0)
   if (!Platform::fileExists(fname))
     return NULL;
 
-  cairo_surface_t *sf = cairo_image_surface_create_from_png(fname.z());
-  assert(cairo_image_surface_get_format(sf) == CAIRO_FORMAT_ARGB32);
-  int wd = cairo_image_surface_get_width(sf);
-  int ht = cairo_image_surface_get_height(sf);
-  int stride = cairo_image_surface_get_stride(sf);
+  WString wname(fname);
+  Gdiplus::Bitmap *bitmap = Gdiplus::Bitmap::FromFile(wname, FALSE);
+  if (bitmap->GetLastStatus() != Gdiplus::Ok) {
+    delete bitmap;
+    return NULL;
+  }
+
+  int wd = bitmap->GetWidth();
+  int ht = bitmap->GetHeight();
+
+  Gdiplus::BitmapData* bitmapData = new Gdiplus::BitmapData;
+  Gdiplus::Rect rect(0, 0, wd, ht);
+  bitmap->LockBits(&rect, Gdiplus::ImageLockModeRead,
+		   PixelFormat32bppPARGB, bitmapData);
 
   if (w < 0) w = wd;
   if (h < 0) h = ht;
@@ -99,11 +110,11 @@ static HBITMAP loadIcon(String action, int w, int h, int r0, int g0, int b0)
   bmi.bmiHeader.biBitCount = (r0 >= 0) ? 24 : 32;
   bmi.bmiHeader.biCompression = BI_RGB;
   void *pbits = 0;
-  HBITMAP bm = CreateDIBSection(NULL, &bmi, DIB_RGB_COLORS,
-				&pbits, NULL, 0);
+  HBITMAP bm = CreateDIBSection(NULL, &bmi, DIB_RGB_COLORS, &pbits, NULL, 0);
 
-  unsigned char *p = cairo_image_surface_get_data(sf);
+  unsigned char *p = (unsigned char *) bitmapData->Scan0;
   unsigned char *q = (unsigned char *) pbits;
+  int stride = bitmapData->Stride;
 
   if (r0 < 0) {
     // make it transparent
@@ -129,17 +140,19 @@ static HBITMAP loadIcon(String action, int w, int h, int r0, int g0, int b0)
       for (int y = 0; y < h; ++y) {
 	unsigned char *src = p + (h - 1 - y) * stride + 4 * x;
 	unsigned char *dst = q + y * dstride + 3 * x;
-	int r1 = *src++;
-	int g1 = *src++;
 	int b1 = *src++;
+	int g1 = *src++;
+	int r1 = *src++;
 	int trans = *src++;
-	*dst++ = r1 + int((0xff - trans) * r0 / 0xff);
-	*dst++ = g1 + int((0xff - trans) * g0 / 0xff);
 	*dst++ = b1 + int((0xff - trans) * b0 / 0xff);
+	*dst++ = g1 + int((0xff - trans) * g0 / 0xff);
+	*dst++ = r1 + int((0xff - trans) * r0 / 0xff);
       }
     }
   }
-  cairo_surface_destroy(sf);
+  bitmap->UnlockBits(bitmapData);
+  delete bitmapData;
+  delete bitmap;
   return bm;
 }
 
@@ -159,7 +172,6 @@ static HBITMAP loadButtonIcon(String action)
   int r0 = GetRValue(rgb);
   int g0 = GetGValue(rgb);
   int b0 = GetBValue(rgb);
-
   return loadIcon(action, -1, -1, r0, g0, b0);
 }
 
@@ -222,22 +234,16 @@ void AppUi::addItem(HMENU menu, const char *title, const char *name)
     lua_getglobal(L, "shortcuts");
     lua_getfield(L, -1, name);
     String sc;
-    if (lua_isstring(L, -1)) {
+    if (lua_isstring(L, -1))
       sc = lua_tostring(L, -1);
-    } else if (lua_istable(L, -1) && lua_rawlen(L, -1) > 0) {
-      lua_rawgeti(L, -1, 1);
-      if (lua_isstring(L, -1))
-	sc = lua_tostring(L, -1);
-      lua_pop(L, 1); // pop string
-    }
     lua_pop(L, 2); // shortcuts, shortcuts[name]
 
     String tooltip = title;
-    if (!sc.isEmpty())
+    if (!sc.empty())
       tooltip = tooltip + " [" + sc + "]";
     createAction(name, tooltip, canUseWhileDrawing);
 
-    if (sc.isEmpty()) {
+    if (sc.empty()) {
       WString wtitle(title);
       AppendMenuW(menu, MF_STRING, iActions.size() - 1 + IDBASE, wtitle);
     } else {
@@ -257,7 +263,7 @@ void AppUi::addItem(int id, const char *title, const char *name)
 
 static HMENU submenu = 0;
 
-void AppUi::startSubMenu(int id, const char *name)
+void AppUi::startSubMenu(int id, const char *name, int tag)
 {
   submenu = CreatePopupMenu();
   WString wname(name);
@@ -738,8 +744,8 @@ void AppUi::layoutChildren(bool resizeRebar)
     MoveWindow(hBookmarksGroup, rc.right - 200, nty, 200, nth, TRUE);
     MoveWindow(hBookmarks, rc.right - 194, nty + 18, 186, nth - 24, TRUE);
   }
-  int statwidths[] = {rc.right - 200, rc.right - 80, rc.right };
-  SendMessage(hStatusBar, SB_SETPARTS, 3, (LPARAM) statwidths);
+  int statwidths[] = {rc.right - 300, rc.right - 200, rc.right - 80, rc.right };
+  SendMessage(hStatusBar, SB_SETPARTS, 4, (LPARAM) statwidths);
   iCanvas->update();
 }
 
@@ -910,10 +916,10 @@ void AppUi::setNumbers(String vno, bool vm, String pno, bool pm)
   setWindowText(hPageNumber, pno.z());
   Button_SetCheck(hViewMarked, vm ? BST_CHECKED : BST_UNCHECKED);
   Button_SetCheck(hPageMarked, pm ? BST_CHECKED : BST_UNCHECKED);
-  EnableWindow(hViewNumber, !vno.isEmpty());
-  EnableWindow(hViewMarked, !vno.isEmpty());
-  EnableWindow(hPageNumber, !pno.isEmpty());
-  EnableWindow(hPageMarked, !pno.isEmpty());
+  EnableWindow(hViewNumber, !vno.empty());
+  EnableWindow(hViewMarked, !vno.empty());
+  EnableWindow(hPageNumber, !pno.empty());
+  EnableWindow(hPageMarked, !pno.empty());
 }
 
 void AppUi::setNotes(String notes)
@@ -1012,6 +1018,27 @@ void AppUi::populateTextStyleMenu()
 		     TEXT_STYLE_BASE + check, MF_BYCOMMAND);
 }
 
+void AppUi::populateSizeMenu(HMENU h, int sel, int base)
+{
+  clearMenu(h);
+  auto i = 0;
+  for (auto &name : iComboContents[sel]) {
+    WString ws(name);
+    AppendMenuW(h, MF_STRING, base + i, ws);
+    ++i;
+  }
+  auto cur = SendMessage(hSelector[sel], CB_GETCURSEL, 0, 0);
+  if (cur != CB_ERR)
+    CheckMenuRadioItem(h, base, base + iComboContents[sel].size() - 1,
+		       base + cur, MF_BYCOMMAND);
+}
+
+void AppUi::populateSizeMenus()
+{
+  populateSizeMenu(iGridSizeMenu, EUiGridSize, ID_GRIDSIZE_BASE);
+  populateSizeMenu(iAngleSizeMenu, EUiAngleSize, ID_ANGLESIZE_BASE);
+}
+
 void AppUi::populateLayerMenus()
 {
   clearMenu(iSelectLayerMenu);
@@ -1032,6 +1059,11 @@ int AppUi::findAction(const char *name) const
   return -1;
 }
 
+int AppUi::actionId(const char *name) const
+{
+  return findAction(name) + IDBASE;
+}
+
 int AppUi::iconId(const char *name) const
 {
   int i = findAction(name);
@@ -1041,16 +1073,13 @@ int AppUi::iconId(const char *name) const
     return I_IMAGENONE;
 }
 
-int AppUi::actionId(const char *name) const
+int AppUi::actionInfo(lua_State *L) const
 {
-  int i = findAction(name);
-  return (i >= 0) ? i + IDBASE : 0;
-}
-
-int AppUi::actionAlwaysOn(const char *name) const
-{
-  int i = findAction(name);
-  return (i >= 0) ? iActions[i].alwaysOn : 0;
+  const char *action = luaL_checkstring(L, 2);
+  int i = findAction(action);
+  lua_pushinteger(L, (i >= 0) ? i + IDBASE : 0);
+  lua_pushboolean(L, (i >= 0) ? iActions[i].alwaysOn : 0);
+  return 2;
 }
 
 void AppUi::cmd(int id, int notification)
@@ -1070,6 +1099,13 @@ void AppUi::cmd(int id, int notification)
   } else if (ID_MOVETOLAYER_BASE <= id &&
 	     id < ID_MOVETOLAYER_BASE + int(iLayerNames.size())) {
     action(String("movetolayer-") + iLayerNames[id - ID_MOVETOLAYER_BASE]);
+  } else if (ID_GRIDSIZE_BASE <= id &&
+	     id < ID_GRIDSIZE_BASE + int(iComboContents[EUiGridSize].size())) {
+    luaSelector("gridsize", iComboContents[EUiGridSize][id - ID_GRIDSIZE_BASE]);
+  } else if (ID_ANGLESIZE_BASE <= id && id < ID_ANGLESIZE_BASE +
+	     int(iComboContents[EUiAngleSize].size())) {
+    luaSelector("anglesize",
+		iComboContents[EUiAngleSize][id - ID_ANGLESIZE_BASE]);
   } else if (id == ID_PATHVIEW) {
     luaShowPathStylePopup(Vector(iPathView->popupPos().x,
 				 iPathView->popupPos().y));
@@ -1083,17 +1119,16 @@ void AppUi::cmd(int id, int notification)
 
 static const char * const aboutText =
   "Ipe %d.%d.%d\n\n"
-  "Copyright (c) 1993-2015 Otfried Cheong\n\n"
+  "Copyright (c) 1993-2016 Otfried Cheong\n\n"
   "The extensible drawing editor Ipe creates figures "
   "in Postscript and PDF format, "
   "using LaTeX to format the text in the figures.\n"
   "Ipe relies on the following fine pieces of software:\n\n"
-  " * Pdflatex\n"
+  " * Pdftex, Xetex, or Luatex\n"
   " * %s (%d kB used)\n" // Lua
   " * The font rendering library %s\n"
   " * The rendering library Cairo %s\n"
-  " * The compression library zlib %s\n"
-  " * Some code from Xpdf\n\n"
+  " * The compression library zlib %s\n\n"
   "Ipe is released under the GNU Public License.\n"
   "See http://ipe.otfried.org for details.";
 
@@ -1135,10 +1170,10 @@ void AppUi::action(String name)
     int i = name.find("|");
     if (i >= 0)
       setCheckMark(name.left(i+1), name.substr(i+1));
-    if (name.left(5) == "mode_")
+    if (name.hasPrefix("mode_"))
       setCheckMark("mode_", name.substr(5));
-    if (name.left(4) == "snap" || name == "grid_visible" ||
-	name == "pretty_display" || name.left(7) == "toggle_")
+    if (name.hasPrefix("snap") || name == "grid_visible" ||
+	name == "pretty_display" || name.hasPrefix("toggle_"))
       setActionState(name.z(), !GetMenuState(hMenuBar, id, MF_CHECKED));
     luaAction(name);
   }
@@ -1171,17 +1206,22 @@ void AppUi::explain(const char *s, int t)
   sendMessage(hStatusBar, SB_SETTEXT, s);
 }
 
-void AppUi::setMouseIndicator(const char *s)
+void AppUi::setSnapIndicator(const char *s)
 {
   sendMessage(hStatusBar, SB_SETTEXT, s, 1);
 }
 
+void AppUi::setMouseIndicator(const char *s)
+{
+  sendMessage(hStatusBar, SB_SETTEXT, s, 2);
+}
+
 void AppUi::setZoom(double zoom)
 {
   char s[32];
   sprintf(s, "%dppi", int(72.0 * zoom));
   iCanvas->setZoom(zoom);
-  sendMessage(hStatusBar, SB_SETTEXT, s, 2);
+  sendMessage(hStatusBar, SB_SETTEXT, s, 3);
 }
 
 // --------------------------------------------------------------------
@@ -1336,6 +1376,8 @@ LRESULT CALLBACK AppUi::wndProc(HWND hwnd, UINT message, WPARAM wParam,
     if (ui) {
       if (lParam == 2)
 	ui->populateTextStyleMenu();
+      else if (lParam == 3)
+	ui->populateSizeMenus();
       else if (lParam == 6)
 	ui->populateLayerMenus();
     }
