@@ -4,7 +4,7 @@
 --[[
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -67,17 +67,15 @@ function MODEL:paction(a)
       for i,e in ipairs(MODEL.snapmodes) do
 	self.ui:setActionState(e, self.snap[e])
       end
-      self.ui:setSnap(self.snap)
+      self:setSnap()
     end
     if prefs.no_ink_highlight then
       self.ui:setFifiVisible(self.mode ~= "ink")
       self.ui:setSelectionVisible(self.mode ~= "ink")
     end
-    -- TODO: This seems redundant?
-    self.ui:setCursor()
   elseif a:sub(1,4) == "snap" then
     self.snap[a] = self.ui:actionState(a)
-    self.ui:setSnap(self.snap)
+    self:setSnap()
     self.ui:setFifiVisible(true)
   elseif a:sub(1,7) == "ipelet_" then
     self:action_ipelet(a)
@@ -107,8 +105,9 @@ function MODEL:selector(prop, value)
   if prop == "gridsize" or prop == "anglesize" then
     local abs = self.doc:sheets():find(prop, value)
     self.snap[prop] = abs
-    self.ui:setSnap(self.snap)
+    self:setSnap()
     self:setPage()
+    self.ui:setGridAngleSize(self.snap.gridsize, self.snap.anglesize)
     return
   end
   if prop == "markshape" then
@@ -652,7 +651,7 @@ end
 
 function MODEL:action_save_as()
   local dir
-  if self.file_name then dir = self.file_name:match("^(.+)/[^/]+") end
+  if self.file_name then dir = self.file_name:match(prefs.dir_pattern) end
   if not dir then dir = prefs.save_as_directory end
   local name
   local filter
@@ -660,14 +659,14 @@ function MODEL:action_save_as()
     name = self.file_name
     local fmt = formatFromFileName(self.file_name)
     if fmt then
-      filter = indexOf(fmt, { "xml", "pdf", "eps" })
+      filter = indexOf(fmt, { "xml", "pdf" })
       name = self.file_name:sub(1,-5)
     end
   end
   local s, f =
     ipeui.fileDialog(self.ui:win(), "save", "Save file as",
 		     filter_save, dir, name, filter)
-  local fmap = { ".ipe", ".pdf", ".eps" }
+  local fmap = { ".ipe", ".pdf" }
   if s then
     if not formatFromFileName(s) then
       s = s .. fmap[f]
@@ -714,12 +713,58 @@ function MODEL:compute_rect(w, h, res)
   return rect
 end
 
+function MODEL:action_export_png()
+  local dir
+  if self.file_name then dir = self.file_name:match(prefs.dir_pattern) end
+  if not dir then dir = prefs.save_as_directory end
+  local s, f =
+    ipeui.fileDialog(self.ui:win(), "save", "Export to PNG", filter_png, dir)
+  if s then
+    if ipe.fileExists(s) then
+      local b = messageBox(self.ui:win(), "question",
+			   "File already exists!",
+			   "Do you wish to overwrite?\n\n" .. s,
+			   "okcancel")
+      if b ~= 1 then
+	self.ui:explain("Document not exported")
+	return
+      end
+    end
+    self.ui:renderPage(self.doc, self.pno, self.vno,
+		       "png", s, self.ui:zoom(),
+		       false, false) -- transparent, nocrop
+  end
+end
+
+function MODEL:action_export_eps()
+  local dir
+  if self.file_name then dir = self.file_name:match(prefs.dir_pattern) end
+  if not dir then dir = prefs.save_as_directory end
+  local s, f =
+    ipeui.fileDialog(self.ui:win(), "save", "Export to EPS", filter_eps, dir)
+  if s then
+    if ipe.fileExists(s) then
+      local b = messageBox(self.ui:win(), "question",
+			   "File already exists!",
+			   "Do you wish to overwrite?\n\n" .. s,
+			   "okcancel")
+      if b ~= 1 then
+	self.ui:explain("Document not exported")
+	return
+      end
+    end
+    self.ui:renderPage(self.doc, self.pno, self.vno,
+		       "eps", s, 1.0, false, false)
+  end
+end
+
 function MODEL:action_insert_image()
   local s, f = ipeui.fileDialog(self.ui:win(), "open", "Insert image",
 				filter_images)
   if s then
-    local format = "jpeg"
-    if s:sub(-4):lower() == ".png" then format = "png" end
+    local format = "png"
+    if s:sub(-4):lower() == ".jpg" or s:sub(-5):lower() == ".jpeg"
+    then format = "jpeg" end
     local bitmap, res = ipe.readImage(s, format)
     if not bitmap then
       messageBox(self.ui:win(), "warning", "Could not read image",
@@ -759,8 +804,8 @@ function MODEL:action_show_configuration()
   s = s .. "\n * Save-as directory: " .. prefs.save_as_directory
   s = s .. "\n * Documentation: " .. config.docdir
   s = s .. "\n * Ipelets:\n   - " .. table.concat(config.ipeletDirs, "\n   - ")
+  s = s .. "\n * Latex program path: " .. config.latexpath
   s = s .. "\n * Latex directory: " .. config.latexdir
-  s = s .. "\n * Fontmap: " .. config.fontmap
   s = s .. "\n * Icons: " .. config.icons
   s = s .. "\n * External editor: " .. (prefs.external_editor or "none")
   messageBox(self.ui:win(), "information", "Ipe configuration", s)
@@ -783,11 +828,11 @@ end
 function MODEL:action_keyboard()
   local s = prefs.keyboard
   if s then
-      if ipeui.startBrowser then
-	ipeui.startBrowser(s)
-      else
-	os.execute(s)
-      end
+    if config.platform == "win" then
+      ipeui.startBrowser(s)
+    else
+      os.execute(s)
+    end
   else
     self:warning("No onscreen keyboard defined.",
 		 "Edit preferences to define an onscreen keyboard.")
@@ -831,7 +876,7 @@ function MODEL:action_set_origin_snap()
   self.snap.snapangle = true
   self.snap.with_axes = true
   self.ui:setActionState("snapangle", true)
-  self.ui:setSnap(self.snap)
+  self:setSnap()
   self.ui:setFifiVisible(true)
   self.ui:update()
 end
@@ -840,7 +885,7 @@ function MODEL:action_hide_axes()
   self.snap.with_axes = false
   self.snap.snapangle = false
   self.ui:setActionState("snapangle", false)
-  self.ui:setSnap(self.snap)
+  self:setSnap()
   self.ui:update()
 end
 
@@ -853,7 +898,7 @@ end
 
 function MODEL:action_reset_direction()
   self.snap.orientation = 0
-  self.ui:setSnap(self.snap)
+  self:setSnap()
   self.ui:update()
 end
 
@@ -873,7 +918,7 @@ end
 
 function MODEL:action_set_line()
   if self:setLine() then
-    self.ui:setSnap(self.snap)
+    self:setSnap()
   end
 end
 
@@ -882,7 +927,7 @@ function MODEL:action_set_line_snap()
     self.snap.snapangle = true
     self.ui:setFifiVisible(true)
     self.ui:setActionState("snapangle", true)
-    self.ui:setSnap(self.snap)
+    self:setSnap()
   end
 end
 
@@ -963,7 +1008,7 @@ end
 
 function MODEL:action_grid_visible()
   self.snap.grid_visible = self.ui:actionState("grid_visible");
-  self.ui:setSnap(self.snap)
+  self:setSnap()
   self.ui:update()
 end
 
@@ -986,16 +1031,14 @@ function MODEL:action_zoom_out()
   self.ui:update()
 end
 
-function MODEL:wheel_zoom(delta)
+function MODEL:wheel_zoom(delta, precise)
+  print("wheel_zoom", delta, precise)
   local origin = self.ui:unsnappedPos()
   local zoom = self.ui:zoom()
   local offset = zoom * (self.ui:pan() - origin)
-  local nzoom
-  if delta > 0 then
-      nzoom = prefs.wheel_zoom_factor * zoom
-  else
-    nzoom = zoom / prefs.wheel_zoom_factor
-  end
+  local base = prefs.wheel_zoom_factor
+  if not precise then base = prefs.wheel_other_factor end
+  local nzoom = zoom * base ^ delta
   if nzoom > prefs.max_zoom then nzoom = prefs.max_zoom end
   if nzoom < prefs.min_zoom then nzoom = prefs.min_zoom end
   self.ui:setZoom(nzoom)
@@ -1003,14 +1046,6 @@ function MODEL:wheel_zoom(delta)
   self.ui:update()
 end
 
-function MODEL:action_wheel_zoom_out()
-  self:wheel_zoom(-1)
-end
-
-function MODEL:action_wheel_zoom_in()
-  self:wheel_zoom(1)
-end
-
 -- Change resolution to 72 dpi and maximize interesting visible area.
 -- As suggested by Rene:
 
@@ -1469,6 +1504,7 @@ function MODEL:action_edit_notes()
   d:addButton("ok", "&Ok", "accept")
   d:addButton("cancel", "&Cancel", "reject")
   d:setStretch("row", 2, 1)
+  d:setStretch("column", 1, 1)
   -- setup original values
   local n = self:page():notes()
   d:set("notes", n)
@@ -1527,6 +1563,7 @@ function MODEL:action_page_sorter()
     self:warning("You cannot delete all pages of the document")
     return
   end
+  printTable(arr)
   -- compute reverse 'permutation'
   local rev = {}
   for i = 1,#self.doc do rev[i] = 0 end
@@ -1548,10 +1585,12 @@ end
 
 function MODEL:action_toggle_notes()
   self.ui:showTool("notes", self.ui:actionState("toggle_notes"))
+  self.ui:setNotes(self:page():notes())
 end
 
 function MODEL:action_toggle_bookmarks()
   self.ui:showTool("bookmarks", self.ui:actionState("toggle_bookmarks"))
+  self:setBookmarks()
 end
 
 ----------------------------------------------------------------------
@@ -2122,7 +2161,7 @@ local function sheets_save(d, dd)
   if s then
     if s:sub(-4) ~= ".isy" then s = s .. ".isy" end
     local data = dd.list[i]:xml(true)
-    local f, msg = io.open(ipeui.local8bit(s), "wb")
+    local f, msg = ipe.openFile(s, "wb")
     if not f then
       dd.model:warning("Cannot save stylesheet", msg)
       return
@@ -2183,6 +2222,7 @@ end
 ----------------------------------------------------------------------
 
 function MODEL:action_document_properties()
+  local engines = { "default", "pdftex", "xetex" } -- "luatex" }
   local p = self.doc:properties()
   local d = ipeui.Dialog(self.ui:win(), "Ipe document properties")
   d:add("label1", "label", { label="Title"}, 1, 1)
@@ -2193,21 +2233,23 @@ function MODEL:action_document_properties()
   d:add("subject", "input", {}, 3, 2, 1, 6)
   d:add("label4", "label", { label="Keywords"}, 4, 1)
   d:add("keywords", "input", {}, 4, 2, 1, 6)
-  d:add("label5", "label", { label="Latex preamble"}, 5, 1)
-  d:add("preamble", "text", {}, 5, 2, 2, 6)
-  d:add("label6", "label", { label="Page mode"}, 7, 1)
-  d:add("fullscreen", "checkbox", { label="&Full screen"}, 7, 2)
-  d:add("numberpages", "checkbox", { label="&Number pages"}, 7, 3)
-  d:add("label7", "label", { label="Created"}, 8, 1)
-  d:add("created", "label",  { label="" }, 8, 2)
-  d:add("label8", "label", { label="Modified"}, 9, 1)
-  d:add("modified", "label",  { label="" }, 9, 2)
-  d:add("label9", "label", { label="Creator" }, 10, 1)
-  d:add("creator", "label",  { label="" }, 10, 2)
+  d:add("label5", "label", { label="Latex engine"}, 5, 1)
+  d:add("tex", "combo", engines, 5, 2, 1, 6)
+  d:add("label6", "label", { label="Latex preamble"}, 6, 1)
+  d:add("preamble", "text", {}, 6, 2, 2, 6)
+  d:add("label7", "label", { label="Page mode"}, 8, 1)
+  d:add("fullscreen", "checkbox", { label="&Full screen"}, 8, 2)
+  d:add("numberpages", "checkbox", { label="&Number pages"}, 8, 3)
+  d:add("label8", "label", { label="Created"}, 9, 1)
+  d:add("created", "label",  { label="" }, 9, 2)
+  d:add("label9", "label", { label="Modified"}, 10, 1)
+  d:add("modified", "label",  { label="" }, 10, 2)
+  d:add("label10", "label", { label="Creator" }, 11, 1)
+  d:add("creator", "label",  { label="" }, 11, 2)
   d:addButton("ok", "&Ok", "accept")
   d:addButton("cancel", "&Cancel", "reject")
   d:setStretch("column", 5, 1)
-  d:setStretch("row", 6, 1)
+  d:setStretch("row", 7, 1)
   for n in pairs(p) do d:set(n, p[n]) end
   if not d:execute() then return end
 
@@ -2218,6 +2260,8 @@ function MODEL:action_document_properties()
   for n in pairs(p) do
     if n == "creator" or n == "created" or n == "modified" then
       t.final[n] = p[n]
+    elseif n == "tex" then
+      t.final[n] = engines[d:get(n)]
     else
       t.final[n] = d:get(n)
     end
