@@ -4,7 +4,7 @@
 --[[
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -31,7 +31,7 @@
 MODEL = {}
 MODEL.__index = MODEL
 
-MODEL.snapmodes = { "snapvtx", "snapbd", "snapint", "snapgrid",
+MODEL.snapmodes = { "snapvtx", "snapctl", "snapbd", "snapint", "snapgrid",
 		    "snapangle", "snapauto"}
 
 function MODEL:new(fname)
@@ -74,6 +74,7 @@ function MODEL:init(fname)
 
   self.snap = {
     snapvtx = prefs.snap.vertex,
+    snapctl = prefs.snap.ctrlpoints,
     snapbd = prefs.snap.boundary,
     snapint = prefs.snap.intersection,
     snapgrid = prefs.snap.grid,
@@ -94,6 +95,23 @@ function MODEL:init(fname)
   self.pristine = false
   self.first_show = true
 
+  self.auto_latex = prefs.auto_run_latex
+  self.ui:setActionState("auto_latex", self.auto_latex)
+
+  self.mode = "select"
+  self.ui:setActionState("mode_select", true)
+  self.ui:setActionState("grid_visible", self.snap.grid_visible)
+
+  for i,e in ipairs(MODEL.snapmodes) do
+    self.ui:setActionState(e, self.snap[e])
+  end
+  self:setSnap()
+  self.ui:setFifiVisible(true)
+
+  for i, dock in ipairs(prefs.tools.hidden) do
+    self.ui:showTool(dock, false)
+  end
+
   local err = nil
   if fname then
     if ipe.fileExists(fname) then
@@ -109,27 +127,6 @@ function MODEL:init(fname)
     self.pristine = true
   end
 
-  self.mode = "select"
-  self.ui:setActionState("mode_select", true)
-  self.ui:setActionState("grid_visible", self.snap.grid_visible)
-
-  for i,e in ipairs(MODEL.snapmodes) do
-    self.ui:setActionState(e, self.snap[e])
-  end
-  self.ui:setSnap(self.snap)
-  self.ui:setFifiVisible(true)
-
-  self.auto_latex = prefs.auto_run_latex
-  self.ui:setActionState("auto_latex", self.auto_latex)
-
-  if self.auto_latex then
-    self:runLatex()
-  end
-
-  for i, dock in ipairs(prefs.hide_docks) do
-    self.ui:showTool(dock, false)
-  end
-
   if prefs.autosave_interval then
     self.timer = ipeui.Timer(self, "autosave")
     self.timer:setInterval(1000 * prefs.autosave_interval) -- millisecs
@@ -249,6 +246,25 @@ function MODEL:setCaption()
   self.ui:setWindowTitle(self:isModified(), s)
 end
 
+function MODEL:setSnap()
+  local function ind(is_on, s, letter)
+    if is_on then return s .. letter
+    else return s .. "-" end
+  end
+
+  self.ui:setSnap(self.snap)
+  local s = ind(self.snap.snapvtx, "", "v")
+  s = ind(self.snap.snapctl, s, "c")
+  s = ind(self.snap.snapbd, s, "b")
+  s = ind(self.snap.snapint, s, "x")
+  s = ind(self.snap.snapgrid, s, "+")
+  s = ind(self.snap.snapangle, s, "r")
+  s = ind(self.snap.snapauto, s, "a")
+  s = s .. " " .. self.snap.gridsize
+  s = s .. " " .. self.snap.anglesize
+  self.ui:setSnapIndicator(s)
+end
+
 -- show a warning messageBox
 function MODEL:warning(text, details)
   messageBox(self.ui:win(), "warning", text, details)
@@ -444,7 +460,7 @@ end
 function MODEL:runLatex()
   local success, errmsg, result, log = self.doc:runLatex()
   if success then
-    self.ui:setFontPool(self.doc)
+    self.ui:setResources(self.doc)
     self.ui:update()
     return true
   elseif result == "latex" then
@@ -490,6 +506,9 @@ function MODEL:newDocument()
     local sheet = assert(ipe.Sheet(w))
     self.doc:sheets():insert(1, sheet)
   end
+  local p = self.doc:properties()
+  p.tex = prefs.tex_engine
+  self.doc:setProperties(p)
 
   self:setPage()
 
@@ -498,7 +517,7 @@ function MODEL:newDocument()
   self:resetGridSize()
 
   self:action_normal_size()
-  self.ui:setSnap(self.snap)
+  self:setSnap()
 
   self.ui:explain("New document")
 end
@@ -530,10 +549,14 @@ function MODEL:tryLoadDocument(fname)
     self:resetGridSize()
 
     self:action_normal_size()
-    self.ui:setSnap(self.snap)
+    self:setSnap()
 
     self.ui:explain("Document '" .. fname .. "' loaded")
 
+    if self.auto_latex then
+      self:runLatex()
+    end
+
     local syms = self.doc:checkStyle()
     if #syms > 0 then
       self:warning("The document contains symbolic attributes " ..
@@ -553,49 +576,18 @@ function MODEL:saveDocument(fname)
   local fm = formatFromFileName(fname)
   if not fm then
     self:warning("File not saved!",
-		 "You must save as *.xml, *.ipe, *.pdf, or *.eps")
-    return
-  end
-
-  if fm == "eps" and self.doc:countTotalViews() > 1 then
-    self:warning("File not saved!",
-		 "Only single-view documents can be saved in EPS format")
+		 "You must save as *.xml, *.ipe, or *.pdf")
     return
   end
 
   -- run Latex if format is not XML
-  if (fm ~= "xml" or prefs.auto_export_xml_to_eps or
-      prefs.auto_export_xml_to_pdf) then
+  if fm ~= "xml" then
     if not self:runLatex() then
       self.ui:explain("Latex error - file not saved")
       return
     end
   end
 
-  if fm == "eps" then
-    local ok = true
-    local msg = "The document uses the following features:\n"
-    if self.doc:has("truetype") then
-      ok = false
-      msg = msg .. "* Truetype fonts\n"
-    end
-    if self.doc:has("gradients") then
-      ok = false
-      msg = msg .. "* Gradients\n"
-    end
-    if self.doc:has("transparency") then
-      ok = false
-      msg = msg .. "* Transparency\n"
-    end
-    if not ok then
-      msg = msg .. "\nThese cannot be saved in EPS format.\n"
-	.. "You can save as PDF, and use pdftops to create a "
-	.. "Postscript file."
-      self:warning("File not saved!", msg)
-      return
-    end
-  end
-
   local props = self.doc:properties()
   props.modified = "D:" .. ipeui.currentDateTime()
   if props.created == "" then props.created = props.modified end
@@ -607,20 +599,6 @@ function MODEL:saveDocument(fname)
     return
   end
 
-  -- auto-exporting
-  if fm == "xml" and prefs.auto_export_xml_to_eps then
-    self:auto_export("eps", fname)
-  end
-  if fm == "xml" and prefs.auto_export_xml_to_pdf then
-    self:auto_export("pdf", fname)
-  end
-  if fm == "eps" and prefs.auto_export_eps_to_pdf then
-    self:auto_export("pdf", fname)
-  end
-  if fm == "pdf" and prefs.auto_export_pdf_to_eps then
-    self:auto_export("eps", fname)
-  end
-
   self:markAsUnmodified()
   self.ui:explain("Saved document '" .. fname .. "'")
   self.file_name = fname
@@ -628,24 +606,6 @@ function MODEL:saveDocument(fname)
   return true
 end
 
-function MODEL:auto_export(fm, fname)
-  local ename = fname:sub(1,-4) .. fm
-  if prefs.auto_export_only_if_exists and not ipe.fileExists(ename) then
-    return
-  end
-  local result
-  if fm == "eps" then
-    result = self.doc:exportView(ename, fm, nil, 1, 1)
-  else
-    result = self.doc:save(ename, fm, { export=true } )
-  end
-  if not result then
-    self:warning("Auto-exporting failed",
-		 "I could not export in format '" .. fm ..
-		   "' to file '" .. ename .. "'.")
-  end
-end
-
 ----------------------------------------------------------------------
 
 function MODEL:autosave()
@@ -675,9 +635,9 @@ function MODEL:closeEvent()
   if self == first_model then first_model = nil end
   if self:isModified() then
     local r = messageBox(self.ui:win(), "question",
-				    "The document has been modified",
-				    "Do you wish to save the document?",
-				    "savediscardcancel")
+			 "The document has been modified",
+			 "Do you wish to save the document?",
+			 "savediscardcancel")
     if r == 1 then
       return self:action_save()
     else
