@@ -1,7 +1,7 @@
 --[[
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -31,8 +31,10 @@
 
 function externalEditor(d, field)
   local text = d:get(field)
-  local fname = prefs.editable_textfile
-  if fname == nil then fname = os.tmpname() end
+  local fname = os.tmpname()
+  if prefs.editable_textfile then
+    fname = prefs.editable_textfile()
+  end
   local f = io.open(fname, "w")
   f:write(text)
   f:close()
@@ -200,6 +202,7 @@ function LINESTOOL:mouseButton(button, modifiers, press)
     table.remove(self.t)
     button = 2
   end
+  if modifiers.control and button == 1 then button = 2 end
   if button == 2 then
     if self:last() == SPLINE then self.t[#self.t] = VERTEX end
     self:compute()
@@ -242,7 +245,7 @@ function LINESTOOL:explain()
   else
     s = "Left: add vtx | Del: delete vtx"
   end
-  s = s .. " | Right or Double-Left: final vtx"
+  s = s .. " | Right, Ctrl-Left, or Double-Left: final vtx"
   if self:has_segs(2) then
     s = s .. " | " .. shortcuts_linestool.spline .. ": spline mode"
   end
@@ -290,7 +293,7 @@ function LINESTOOL:key(text, modifiers)
     self.model.snap.snapangle = true
     self.model.snap.origin = self.v[#self.v - 1]
     self.model.snap.orientation = self:compute_orientation()
-    self.model.ui:setSnap(self.model.snap)
+    self.model:setSnap()
     self.model.ui:setActionState("snapangle", true)
     self.model.ui:update(true)   -- redraw coordinate system
     self:explain()
@@ -380,7 +383,8 @@ function SPLINEGONTOOL:compute(update)
 end
 
 function SPLINEGONTOOL:explain()
-  local s = "Left: Add vertex | Right or double-left: Add final vertex"
+  local s = "Left: Add vertex "
+    .. "| Right, ctrl-left, or double-left: Add final vertex"
     .. "| Del: Delete vertex"
   self.model.ui:explain(s, 0)
 end
@@ -396,6 +400,7 @@ function SPLINEGONTOOL:mouseButton(button, modifiers, press)
     table.remove(self.v)
     button = 2
   end
+  if modifiers.control and button == 1 then button = 2 end
   if button == 2 then
     self:compute(false)
     self.model.ui:finishTool()
@@ -711,7 +716,8 @@ function MODEL:createText(mode)
   end
   local d = ipeui.Dialog(self.ui:win(), "Create text object")
   d:add("label", "label", { label=prompt }, 1, 1, 1, 2)
-  d:add("text", "text", { syntax="latex", focus=true }, 2, 1, 1, 2)
+  d:add("text", "text", { syntax="latex", focus=true,
+			  spell_check=prefs.spell_check }, 2, 1, 1, 2)
   addEditorField(d, "text")
   d:addButton("ok", "&Ok", "accept")
   d:addButton("cancel", "&Cancel", "reject")
@@ -742,7 +748,8 @@ function MODEL:createParagraph(pos, width, pinned)
   d:add("label", "label", { label="Enter latex source" }, 1, 1, 1, 2)
   d:add("style", "combo", styles, 1, 3)
   d:add("size", "combo", sizes, 1, 4)
-  d:add("text", "text", { syntax="latex", focus=true }, 2, 1, 1, 4)
+  d:add("text", "text", { syntax="latex", focus=true,
+			  spell_check=prefs.spell_check }, 2, 1, 1, 4)
   addEditorField(d, "text")
   d:addButton("ok", "&Ok", "accept")
   d:addButton("cancel", "&Cancel", "reject")
@@ -959,7 +966,7 @@ local mouse_mappings = {
 
 function MODEL:mouseButtonAction(button, modifiers)
   -- print("Mouse button", button, modifiers.alt, modifiers.control)
-  if button == 0x81 then button = 1 end -- double-click
+  if button == 0x81 then button = 1 end -- left double-click
   if button == 1 and not modifiers.alt and
     not modifiers.control and not modifiers.meta then
     self:startModeTool(modifiers)
@@ -1028,7 +1035,8 @@ function MODEL:action_edit_text(prim, obj)
 		 obj=obj,
 	       }
   d:add("label", "label", { label="Edit latex source" }, 1, 1, 1, 2)
-  d:add("text", "text", { syntax="latex", focus=true }, 2, 1, 1, 4)
+  d:add("text", "text", { syntax="latex", focus=true,
+			  spell_check=prefs.spell_check}, 2, 1, 1, 4)
   d:addButton("apply", "&Apply",
 	      function (d) apply_text_edit(d, data, true) end)
   addEditorField(d, "text")
