@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -34,6 +34,8 @@
 
 #include "ipelua.h"
 
+#include "ipethumbs.h"
+
 #include <cstdio>
 #include <cstdlib>
 
@@ -83,12 +85,12 @@ static int appui_setPage(lua_State *L)
   return 0;
 }
 
-static int appui_setFontPool(lua_State *L)
+static int appui_setResources(lua_State *L)
 {
   CanvasBase *canvas = check_canvas(L, 1);
   Document **d = check_document(L, 2);
-  const FontPool *fonts = (*d)->fontPool();
-  canvas->setFontPool(fonts);
+  const PdfResources *res = (*d)->resources();
+  canvas->setResources(res);
   return 0;
 }
 
@@ -122,6 +124,13 @@ static int appui_setZoom(lua_State *L)
   return 0;
 }
 
+static int appui_setSnapIndicator(lua_State *L)
+{
+  AppUiBase **ui = check_appui(L, 1);
+  (*ui)->setSnapIndicator(luaL_checkstring(L, 2));
+  return 0;
+}
+
 static int appui_pos(lua_State *L)
 {
   CanvasBase *canvas = check_canvas(L, 1);
@@ -233,7 +242,7 @@ static int appui_update(lua_State *L)
     else
       canvas->updateTool();
   } else {
-    /*
+    /* TODO: Is this used anywhere???
     Rect *r = check_rect(L, 2);
     Vector bl = canvas->userToDev(r->topLeft());
     Vector tr = canvas->userToDev(r->bottomRight());
@@ -315,6 +324,7 @@ static int appui_setPretty(lua_State *L)
 
 static int appui_pantool(lua_State *L)
 {
+  ipeDebug("pantool");
   CanvasBase *canvas = check_canvas(L, 1);
   Page *page = check_page(L, 2)->page;
   int view = check_viewno(L, 3, page);
@@ -493,13 +503,11 @@ static int appui_setClipboard(lua_State *L)
   return (*ui)->setClipboard(L);
 }
 
+// This is only used on Windows to compute the shortcuts...
 static int appui_actionInfo(lua_State *L)
 {
   AppUiBase **ui = check_appui(L, 1);
-  const char *action = luaL_checkstring(L, 2);
-  lua_pushinteger(L, (*ui)->actionId(action));
-  lua_pushboolean(L, (*ui)->actionAlwaysOn(action));
-  return 2;
+  return (*ui)->actionInfo(L);
 }
 
 static int appui_actionState(lua_State *L)
@@ -697,6 +705,26 @@ static int appui_pageSorter(lua_State *L)
   return (*ui)->pageSorter(L, *doc, width, height, thumbWidth);
 }
 
+static const char *const render_formats[] = {"svg", "png", "eps", "pdf", NULL };
+
+static int appui_renderPage(lua_State *L)
+{
+  // AppUiBase **ui = check_appui(L, 1);  // not used
+  Document **doc = check_document(L, 2);
+  int pageno = luaL_checkinteger(L, 3);
+  int viewno = luaL_checkinteger(L, 4);
+  Thumbnail::TargetFormat fm =
+    Thumbnail::TargetFormat(luaL_checkoption(L, 5, NULL, render_formats));
+  const char *dst = luaL_checkstring(L, 6);
+  double zoom = luaL_checknumber(L, 7);
+  bool transparent = lua_toboolean(L, 8);
+  bool nocrop = lua_toboolean(L, 9);
+  Thumbnail tn(*doc, 0);
+  const Page *page = (*doc)->page(pageno - 1);
+  tn.saveRender(fm, dst, page, viewno - 1, zoom, transparent, nocrop);
+  return 0;
+}
+
 // --------------------------------------------------------------------
 
 static const struct luaL_Reg appui_methods[] = {
@@ -708,7 +736,7 @@ static const struct luaL_Reg appui_methods[] = {
   { "setPan", appui_setPan},
   { "zoom", appui_zoom},
   { "setZoom", appui_setZoom},
-  { "setFontPool", appui_setFontPool},
+  { "setResources", appui_setResources},
   { "pos", appui_pos},
   { "globalPos", appui_globalPos},
   { "unsnappedPos", appui_unsnappedPos},
@@ -716,6 +744,7 @@ static const struct luaL_Reg appui_methods[] = {
   { "setFifiVisible", appui_setFifiVisible},
   { "setSelectionVisible", appui_setSelectionVisible},
   { "setSnap", appui_setSnap},
+  { "setSnapIndicator", appui_setSnapIndicator},
   { "setAutoOrigin", appui_setAutoOrigin },
   { "update", appui_update},
   { "finishTool", appui_finishTool},
@@ -749,6 +778,7 @@ static const struct luaL_Reg appui_methods[] = {
   { "showTool", appui_showTool },
   { "selectPage", appui_selectPage },
   { "pageSorter", appui_pageSorter },
+  { "renderPage", appui_renderPage },
   { 0, 0},
 };
 
@@ -770,6 +800,8 @@ static int appui_constructor(lua_State *L)
   CanvasBase::Style style;
   style.pretty = false;
   style.paperColor = Color(1000,1000,1000);
+  style.primarySelectionColor = Color(1000,0,0);
+  style.secondarySelectionColor = Color(1000,0,1000);
   style.classicGrid = false;
   style.thinLine = 0.2;
   style.thickLine = 0.9;
@@ -787,6 +819,16 @@ static int appui_constructor(lua_State *L)
       style.paperColor = check_color(L, lua_gettop(L));
     lua_pop(L, 1); // paper_color
 
+    lua_getfield(L, -1, "primary_color");
+    if (!lua_isnil(L, -1))
+      style.primarySelectionColor = check_color(L, lua_gettop(L));
+    lua_pop(L, 1); // primary_color
+
+    lua_getfield(L, -1, "secondary_color");
+    if (!lua_isnil(L, -1))
+      style.secondarySelectionColor = check_color(L, lua_gettop(L));
+    lua_pop(L, 1); // secondary_color
+
     lua_getfield(L, -1, "classic_grid");
     style.classicGrid = lua_toboolean(L, -1);
     lua_pop(L, 1); // classic_grid
