@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -49,7 +49,9 @@ using namespace ipe;
 
 // --------------------------------------------------------------------
 
-void CanvasObserver::canvasObserverWheelMoved(int degrees) { /* nothing */ }
+CanvasObserver::~CanvasObserver() { /* nothing */ }
+void CanvasObserver::canvasObserverWheelMoved(double degrees,
+					      bool precise) { /* nothing */ }
 void CanvasObserver::canvasObserverMouseAction(int button) { /* nothing */ }
 void CanvasObserver::canvasObserverPositionChanged() { /* nothing */ }
 void CanvasObserver::canvasObserverToolChanged(bool hasTool) { /* nothing */ }
@@ -73,6 +75,8 @@ CanvasBase::CanvasBase()
   iDimmed = false;
   iWidth = 0;   // not yet known (canvas is not yet mapped)
   iHeight = 0;
+  iBWidth = 0;
+  iBHeight = 0;
   iRepaintObjects = false;
   iFonts = 0;
   iAutoSnap = false;
@@ -83,6 +87,8 @@ CanvasBase::CanvasBase()
   iAdditionalModifiers = 0;
 
   iStyle.paperColor = Color(1000,1000,1000);
+  iStyle.primarySelectionColor = Color(1000, 0, 0);
+  iStyle.secondarySelectionColor= Color(1000, 0, 1000);
   iStyle.pretty = false;
   iStyle.classicGrid = false;
   iStyle.thinLine = 0.2;
@@ -95,7 +101,7 @@ CanvasBase::CanvasBase()
   iSnap.iSnap = 0;
   iSnap.iGridVisible = false;
   iSnap.iGridSize = 8;
-  iSnap.iAngleSize = M_PI / 6.0;
+  iSnap.iAngleSize = IPE_PI / 6.0;
   iSnap.iSnapDistance = 10;
   iSnap.iWithAxes = false;
   iSnap.iOrigin = Vector::ZERO;
@@ -115,10 +121,10 @@ CanvasBase::~CanvasBase()
 // --------------------------------------------------------------------
 
 //! set information about Latex fonts (from ipe::Document)
-void CanvasBase::setFontPool(const FontPool *fontPool)
+void CanvasBase::setResources(const PdfResources *resources)
 {
   delete iFonts;
-  iFonts = Fonts::New(fontPool);
+  iFonts = Fonts::New(resources);
 }
 
 // --------------------------------------------------------------------
@@ -173,8 +179,8 @@ void CanvasBase::setDimmed(bool dimmed)
 
 //! Set additional modifiers.
 /*! These modifier bits are passed to the Tool when a key is pressed
-    or a drawing action is performed in addition to the actual
-    keyboard modifiers. */
+  or a drawing action is performed in addition to the actual
+  keyboard modifiers. */
 void CanvasBase::setAdditionalModifiers(int mod)
 {
   iAdditionalModifiers = mod;
@@ -210,9 +216,7 @@ Vector CanvasBase::userToDev(const Vector &arg) const
 //! Matrix mapping user coordinates to canvas coordinates
 Matrix CanvasBase::canvasTfm() const
 {
-  return Matrix(center()) *
-    Linear(iZoom, 0, 0, -iZoom) *
-    Matrix(-iPan);
+  return Matrix(center()) * Linear(iZoom, 0, 0, -iZoom) * Matrix(-iPan);
 }
 
 // --------------------------------------------------------------------
@@ -493,11 +497,11 @@ void CanvasBase::drawTool(Painter &painter)
     for (int i = 0; i < iPage->count(); ++i) {
       if (iPage->objectVisible(iView, i)) {
 	if (iPage->select(i) == EPrimarySelected) {
-	  painter.setStroke(Attribute(Color(1000, 0, 0)));
+	  painter.setStroke(Attribute(iStyle.primarySelectionColor));
 	  painter.setPen(Attribute(Fixed(2)));
 	  iPage->object(i)->drawSimple(painter);
 	} else if (iPage->select(i) == ESecondarySelected) {
-	  painter.setStroke(Attribute(Color(1000, 0, 1000)));
+	  painter.setStroke(Attribute(iStyle.secondarySelectionColor));
 	  painter.setPen(Attribute(Fixed(1)));
 	  iPage->object(i)->drawSimple(painter);
 	}
@@ -651,12 +655,15 @@ void CanvasBase::computeFifi(double x, double y)
     // convert fifi coordinates back into device space
     Vector fifi = userToDev(iMousePos);
     if (iFifiVisible && fifi != iOldFifi) {
-      invalidate(int(iOldFifi.x - 10), int(iOldFifi.y - 10), 21, 21);
-      invalidate(int(fifi.x - 10), int(fifi.y - 10), 21, 21);
+      invalidate(int(iOldFifi.x - 10),
+		 int(iOldFifi.y - 10), 21, 21);
+      invalidate(int(fifi.x - 10),
+		 int(fifi.y - 10), 21, 21);
     }
   } else if (iFifiVisible) {
     // remove old fifi
-    invalidate(int(iOldFifi.x - 10), int(iOldFifi.y - 10), 21, 21);
+    invalidate(int(iOldFifi.x - 10),
+	       int(iOldFifi.y - 10), 21, 21);
     iFifiVisible = false;
   }
 }
@@ -666,10 +673,11 @@ void CanvasBase::computeFifi(double x, double y)
 void CanvasBase::refreshSurface()
 {
   if (!iSurface
-      || iWidth != cairo_image_surface_get_width(iSurface)
-      || iHeight != cairo_image_surface_get_height(iSurface)) {
+      || iBWidth != cairo_image_surface_get_width(iSurface)
+      || iBHeight != cairo_image_surface_get_height(iSurface)) {
     // size has changed
-    ipeDebug("size has changed to %d x %d", iWidth, iHeight);
+    ipeDebug("size has changed to %g x %g (%g x %g)",
+	     iWidth, iHeight, iBWidth, iBHeight);
     if (iSurface)
       cairo_surface_destroy(iSurface);
     iSurface = 0;
@@ -681,15 +689,16 @@ void CanvasBase::refreshSurface()
   if (iRepaintObjects) {
     iRepaintObjects = false;
     if (!iSurface)
-      iSurface =
-	cairo_image_surface_create(CAIRO_FORMAT_RGB24, iWidth, iHeight);
+      iSurface = cairo_image_surface_create(CAIRO_FORMAT_RGB24,
+					    iBWidth, iBHeight);
     cairo_t *cc = cairo_create(iSurface);
     // background
     cairo_set_source_rgb(cc, 0.4, 0.4, 0.4);
-    cairo_rectangle(cc, 0, 0, iWidth, iHeight);
+    cairo_rectangle(cc, 0, 0, iBWidth, iBHeight);
     cairo_fill(cc);
 
-    cairo_translate(cc, 0.5 * iWidth, 0.5 * iHeight);
+    cairo_translate(cc, 0.5 * iBWidth, 0.5 * iBHeight);
+    cairo_scale(cc, iBWidth / iWidth, iBHeight / iHeight);
     cairo_scale(cc, iZoom, -iZoom);
     cairo_translate(cc, -iPan.x, -iPan.y);
 
