@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -28,11 +28,9 @@
 
 */
 
-#ifndef IPENOGESTURE
 // for touch gestures:  need at least Windows 7
 #define WINVER 0x0601
 #define _WIN32_WINNT 0x0601
-#endif
 
 #include "ipecanvas_win.h"
 #include "ipecairopainter.h"
@@ -46,6 +44,10 @@
 
 using namespace ipe;
 
+typedef WINBOOL WINAPI (*LPFNGETGESTUREINFO)(HGESTUREINFO hGestureInfo,
+					     PGESTUREINFO pGestureInfo);
+static LPFNGETGESTUREINFO pGetGestureInfo = 0;
+
 const wchar_t Canvas::className[] = L"ipeCanvasWindowClass";
 
 // --------------------------------------------------------------------
@@ -77,7 +79,7 @@ void Canvas::key(WPARAM wParam, LPARAM lParam)
     mod |= EControl;
   if (GetKeyState(VK_MENU) < 0)
     mod |= EAlt;
-  ipeDebug("Canvas::key(%x, %x) %x", wParam, lParam, mod);
+  // ipeDebug("Canvas::key(%x, %x) %x", wParam, lParam, mod);
   String t;
   t.append(char(wParam));
   iTool->key(t, mod);
@@ -85,7 +87,7 @@ void Canvas::key(WPARAM wParam, LPARAM lParam)
 
 void Canvas::button(int button, bool down, int mod, int xPos, int yPos)
 {
-  ipeDebug("Canvas::button %d %d %d %d", button, down, xPos, yPos);
+  // ipeDebug("Canvas::button %d %d %d %d", button, down, xPos, yPos);
   POINT p;
   p.x = xPos;
   p.y = yPos;
@@ -126,8 +128,8 @@ void Canvas::updateSize()
 {
   RECT rc;
   GetClientRect(hwnd, &rc);
-  iWidth = rc.right;
-  iHeight = rc.bottom;
+  iBWidth = iWidth = rc.right;
+  iBHeight = iHeight = rc.bottom;
 }
 
 void Canvas::wndPaint()
@@ -191,7 +193,6 @@ void Canvas::mouseButton(Canvas *canvas, int button, bool down,
   }
 }
 
-#ifndef IPENOGESTURE
 LRESULT Canvas::handlePanGesture(DWORD flags, POINTS &p)
 {
   Vector v(p.x, -p.y);
@@ -222,11 +223,10 @@ LRESULT Canvas::handleZoomGesture(DWORD flags, POINTS &p, ULONG d)
     update();
     if (iObserver)
       // scroll wheel hasn't moved, but update display of ppi
-      iObserver->canvasObserverWheelMoved(0);
+      iObserver->canvasObserverWheelMoved(0, true);
   }
   return 0;
 }
-#endif
 
 LRESULT CALLBACK Canvas::wndProc(HWND hwnd, UINT message, WPARAM wParam,
 				 LPARAM lParam)
@@ -255,7 +255,7 @@ LRESULT CALLBACK Canvas::wndProc(HWND hwnd, UINT message, WPARAM wParam,
   case WM_MOUSEWHEEL:
     if (canvas && canvas->iObserver)
       canvas->iObserver->canvasObserverWheelMoved
-	(GET_WHEEL_DELTA_WPARAM(wParam));
+	(GET_WHEEL_DELTA_WPARAM(wParam) / 120.0, true);
     return 0;
   case WM_PAINT:
     if (canvas)
@@ -265,11 +265,12 @@ LRESULT CALLBACK Canvas::wndProc(HWND hwnd, UINT message, WPARAM wParam,
     if (canvas)
       canvas->updateSize();
     break;
-#ifndef IPENOGESTURE
   case WM_GESTURE: {
+    if (!pGetGestureInfo)
+      break;  // GetGestureInfo not available (Windows Vista or older)
     GESTUREINFO gi = {0};
     gi.cbSize = sizeof(gi);
-    BOOL res = GetGestureInfo((HGESTUREINFO) lParam, &gi);
+    BOOL res = pGetGestureInfo((HGESTUREINFO) lParam, &gi);
     if (res && canvas) {
       if (gi.dwID == GID_PAN)
 	return canvas->handlePanGesture(gi.dwFlags, gi.ptsLocation);
@@ -278,7 +279,6 @@ LRESULT CALLBACK Canvas::wndProc(HWND hwnd, UINT message, WPARAM wParam,
 					 gi.ullArguments);
     }
     break; }
-#endif
   case WM_MOUSEMOVE:
     if (canvas) {
       int xPos = GET_X_LPARAM(lParam);
@@ -361,6 +361,12 @@ void Canvas::init(HINSTANCE hInstance)
 		MB_ICONEXCLAMATION | MB_OK);
     exit(9);
   }
+
+  // Check if GetGestureInfo is available (i.e. at least Windows 7)
+  HMODULE hDll = LoadLibraryA("user32.dll");
+  if (hDll)
+    pGetGestureInfo = (LPFNGETGESTUREINFO)
+      GetProcAddress(hDll, "GetGestureInfo");
 }
 
 // --------------------------------------------------------------------
