@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -29,6 +29,7 @@
 */
 
 #include "ipeui_common.h"
+#include <algorithm>
 
 // --------------------------------------------------------------------
 
@@ -190,14 +191,14 @@ int Dialog::add(lua_State *L)
   luaL_checktype(L, 4, LUA_TTABLE);
   m.lua_method = LUA_NOREF;
   m.flags = 0;
-  m.row = luaL_checkint(L, 5) - 1;
-  m.col = luaL_checkint(L, 6) - 1;
+  m.row = (int)luaL_checkinteger(L, 5) - 1;
+  m.col = (int)luaL_checkinteger(L, 6) - 1;
   m.rowspan = 1;
   m.colspan = 1;
   if (!lua_isnoneornil(L, 7))
-    m.rowspan = luaL_checkint(L, 7);
+    m.rowspan = (int)luaL_checkinteger(L, 7);
   if (!lua_isnoneornil(L, 8))
-    m.colspan = luaL_checkint(L, 8);
+    m.colspan = (int)luaL_checkinteger(L, 8);
   if (m.row + m.rowspan > iNoRows)
     iNoRows = m.row + m.rowspan;
   if (m.col + m.colspan > iNoCols)
@@ -336,7 +337,10 @@ void Dialog::addTextEdit(lua_State *L, SElement &m)
     } else
       luaL_argerror(L, 4, "unknown syntax");
   }
-  lua_pop(L, 4); // syntax, focus, select_all, read_only
+  lua_getfield(L, 4, "spell_check");
+  if (lua_toboolean(L, -1))
+    m.flags |= ESpellCheck;
+  lua_pop(L, 5); // spell_check, syntax, focus, select_all, read_only
   m.minHeight = 48;
   m.minWidth = 100;
 }
@@ -412,10 +416,17 @@ void Dialog::setUnmapped(lua_State *L, int idx)
   case EList:
   case ECombo:
     if (lua_isnumber(L, 3)) {
-      int n = luaL_checkint(L, 3);
+      int n = (int)luaL_checkinteger(L, 3);
       luaL_argcheck(L, 1 <= n && n <= int(m.items.size()), 3,
 		    "list index out of bounds");
       m.value = n - 1;
+    } else if (lua_isstring(L, 3)) {
+      const char *s = lua_tostring(L, 3);
+      auto it = std::find_if(m.items.cbegin(), m.items.cend(),
+			     [s](const String &el)
+			     { return !strcmp(el.z(), s); });
+      luaL_argcheck(L, it != m.items.end(), 3, "item not in list");
+      m.value = (it - m.items.begin());
     } else {
       luaL_checktype(L, 3, LUA_TTABLE);
       m.items.clear();
@@ -492,8 +503,8 @@ int Dialog::setStretch(lua_State *L)
     iRowStretch.push_back(0);
 
   int type = luaL_checkoption(L, 2, 0, typenames);
-  int rowcol = luaL_checkint(L, 3) - 1;
-  int stretch = luaL_checkint(L, 4);
+  int rowcol = (int)luaL_checkinteger(L, 3) - 1;
+  int stretch = (int)luaL_checkinteger(L, 4);
 
   if (type == 0) {
     luaL_argcheck(L, 0 <= rowcol && rowcol < iNoRows, 3,
@@ -583,7 +594,7 @@ static int dialog_setEnabled(lua_State *L)
 static int dialog_accept(lua_State *L)
 {
   Dialog **dlg = check_dialog(L, 1);
-  (*dlg)->accept(L);
+  (*dlg)->acceptDialog(L);
   return 0;
 }
 
@@ -685,8 +696,7 @@ void Timer::callLua()
 
 int Timer::setSingleShot(lua_State *L)
 {
-  bool t = lua_toboolean(L, 2);
-  iSingleShot = t;
+  iSingleShot = lua_toboolean(L, 2);
   return 0;
 }
 
