@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -52,10 +52,6 @@
 #include <QThread>
 #include <QTimer>
 
-#if defined(WIN32)
-#include <windows.h>
-#endif
-
 // --------------------------------------------------------------------
 
 inline void push_string(lua_State *L, const QString &str)
@@ -883,14 +879,6 @@ static int ipeui_messageBox(lua_State *L)
 
 // --------------------------------------------------------------------
 
-static int ipeui_local8bit(lua_State *L)
-{
-  QString data = checkstring(L, 1);
-  QByteArray l8 = data.toLocal8Bit();
-  lua_pushstring(L, l8.data());
-  return 1;
-}
-
 static int ipeui_currentDateTime(lua_State *L)
 {
   QDateTime dt = QDateTime::currentDateTime();
@@ -902,15 +890,6 @@ static int ipeui_currentDateTime(lua_State *L)
   return 1;
 }
 
-#if defined(WIN32)
-static int ipeui_startBrowser(lua_State *L)
-{
-  const char *url = luaL_checkstring(L, 1);
-  ShellExecuteA(0, "open", url, 0, 0, 0);
-  return 0;
-}
-#endif
-
 // --------------------------------------------------------------------
 
 static const struct luaL_Reg ipeui_functions[] = {
@@ -922,10 +901,6 @@ static const struct luaL_Reg ipeui_functions[] = {
   { "messageBox", ipeui_messageBox },
   { "waitDialog", ipeui_wait },
   { "currentDateTime", ipeui_currentDateTime },
-  { "local8bit", ipeui_local8bit },
-#ifdef WIN32
-  { "startBrowser", ipeui_startBrowser },
-#endif
   { 0, 0},
 };
 
