@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -137,7 +137,7 @@ protected:
   void computeDimensions(int &w, int &h);
   void buildDimensions(std::vector<short> &t, SElement &m, int id);
 
-  virtual void accept(lua_State *L);
+  virtual void acceptDialog(lua_State *L);
 
   BOOL initDialog();
   BOOL dlgCommand(WPARAM wParam, LPARAM lParam);
@@ -374,7 +374,7 @@ BOOL PDialog::dlgCommand(WPARAM wParam, LPARAM lParam)
   return FALSE;
 }
 
-void PDialog::accept(lua_State *L)
+void PDialog::acceptDialog(lua_State *L)
 {
   int accept = lua_toboolean(L, 2);
   retrieveValues();
@@ -386,7 +386,7 @@ static LRESULT subclassProc(HWND hwnd, UINT message,
 			    WPARAM wParam, LPARAM lParam)
 {
   if (message == WM_KEYDOWN) {
-    if (wParam == VK_RETURN && GetKeyState(VK_CONTROL)) {
+    if (wParam == VK_RETURN && (GetKeyState(VK_CONTROL) & 0x8000)) {
       SendMessage(GetParent(hwnd), WM_COMMAND, IDOK, 0);
       return 0;
     }
@@ -618,11 +618,13 @@ void PDialog::computeDimensions(int &w, int &h)
       iRowHeight[i] = minHeight[i] + extra;
     }
   }
+  /*
   fprintf(stderr, "iColWidth\n");
   for (int i = 0; i < iNoCols; ++i)
     fprintf(stderr, "%d ", iColWidth[i]);
   fprintf(stderr, "\n");
   fflush(stderr);
+  */
 }
 
 void PDialog::enableItem(int idx, bool value)
@@ -1223,16 +1225,6 @@ static int ipeui_wait(lua_State *L)
 
 // --------------------------------------------------------------------
 
-static int ipeui_local8bit(lua_State *L)
-{
-  WString in(luaL_checkstring(L, 1));
-  int rm = WideCharToMultiByte(CP_ACP, 0, in, -1, NULL, 0, NULL, NULL);
-  char multi[rm];
-  WideCharToMultiByte(CP_ACP, 0, in, -1, multi, rm, NULL, NULL);
-  lua_pushstring(L, multi);
-  return 1;
-}
-
 static int ipeui_currentDateTime(lua_State *L)
 {
   SYSTEMTIME st;
@@ -1264,7 +1256,6 @@ static const struct luaL_Reg ipeui_functions[] = {
   { "messageBox", ipeui_messageBox },
   { "waitDialog", ipeui_wait },
   { "currentDateTime", ipeui_currentDateTime },
-  { "local8bit", ipeui_local8bit },
   { "startBrowser", ipeui_startBrowser },
   { 0, 0},
 };
